/******************************************************************************
* File Name: Main.c
*
* Version 1.1
*
* Description:
* This file contains the main function for the voltage Display test.
*
* Note:
*
* Code tested with:
* PSoC Creator: 2.1
* Device Tested With: CY8C5868AXI-LP035
* Compiler    : ARMGCC 4.4.1, ARM RVDS Generic, ARM MDK Generic
*
********************************************************************************
* Copyright (2012), Cypress Semiconductor Corporation. All Rights Reserved.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*******************************************************************************/

/******************************************************************************
*                           THEORY OF OPERATION
* This project demonstrates how ADC is used to read the input voltage at 
* it's input and display it on the LCD.
* 
* The Potentiometer is connected to the input of the SAR ADC. ADC is 
* configured with 12 bit of resolution to measure the input voltage with 
* higher accuracy. ADC count is converted back to micro volts and 8 samples are 
* collected. After that the average of 8 samples is calculated and displayed
* on the LCD. 
*
* Hardware connection on the Kit
* Potentiometer - PORT 6[5] 
* LCD - PORT 2[0..6]
*******************************************************************************/
#include <device.h>
#include "stdio.h"
#include "math.h"


#if defined (__GNUC__)
    /* Add an explicit reference to the floating point printf library */
    /* to allow the usage of floating point conversion specifiers. */
    /* This is not linked in by default with the newlib-nano library. */
    asm (".global _printf_float");
#endif

/* The size of the buffer is equal to maximum packet size of the 
*  IN and OUT bulk endpoints. 
*/
#define BUFFER_LEN  64u
#define ATAN_ONE 4096

// Get the octant a coordinate pair is in.
#define OCTANTIFY(_x, _y, _o) do {							\
	int _t; _o = 0;											\
	if(_y <   0) {              _x = -_x;     _y = -_y; _o += 4;}	\
	if(_x <=  0) { _t = _x;     _x =  _y;     _y = -_t; _o += 2;}	\
	if(_x <= _y) { _t = _y -_x; _x =  _x +_y; _y =  _t; _o += 1;}	\
} while(0);

static const uint16_t BRAD_PI_SHIFT = 14;
static const uint16_t BRAD_PI = 16384;  //1 << BRAD_PI_SHIFT

static const uint16_t A[130] =
{
  0x0000,0x0146,0x028c,0x03d2,0x0517,0x065d,0x07a2,0x08e7, 
  0x0a2c,0x0b71,0x0cb5,0x0df9,0x0f3c,0x107f,0x11c1,0x1303,
  0x1444,0x1585,0x16c5,0x1804,0x1943,0x1a80,0x1bbd,0x1cfa,
  0x1e35,0x1f6f,0x20a9,0x21e1,0x2319,0x2450,0x2585,0x26ba,
  0x27ed,0x291f,0x2a50,0x2b80,0x2caf,0x2ddc,0x2f08,0x3033,
  0x315d,0x3285,0x33ac,0x34d2,0x35f6,0x3719,0x383a,0x395a,
  0x3a78,0x3b95,0x3cb1,0x3dcb,0x3ee4,0x3ffb,0x4110,0x4224,
  0x4336,0x4447,0x4556,0x4664,0x4770,0x487a,0x4983,0x4a8b,
  //64
  0x4b90,0x4c94,0x4d96,0x4e97,0x4f96,0x5093,0x518f,0x5289,
  0x5382,0x5478,0x556e,0x5661,0x5753,0x5843,0x5932,0x5a1e,
  0x5b0a,0x5bf3,0x5cdb,0x5dc1,0x5ea6,0x5f89,0x606a,0x614a,
  0x6228,0x6305,0x63e0,0x64b9,0x6591,0x6667,0x673b,0x680e,
  0x68e0,0x69b0,0x6a7e,0x6b4b,0x6c16,0x6cdf,0x6da8,0x6e6e,
  0x6f33,0x6ff7,0x70b9,0x717a,0x7239,0x72f6,0x73b3,0x746d,
  0x7527,0x75df,0x7695,0x774a,0x77fe,0x78b0,0x7961,0x7a10,
  0x7abf,0x7b6b,0x7c17,0x7cc1,0x7d6a,0x7e11,0x7eb7,0x7f5c,
  //128
  0x8000,0x80a2	
};


/* Variable to hold ADC count */
    int16 voltCount_cos = 0;
    uint16 result_cos = 0;
    int16 voltCount_sin = 0;
    uint16 result_sin = 0;
    uint8 period = 0;
    uint16 phi_LUT_int;
    uint16  Disp_Flag = 0;
    uint16 phi_alt;
   // uint8 staticBits;


/******************************************************************************
* Function Name: ADC_COS_ISR_LOC
*******************************************************************************
*
* Summary:
*  Handle Interrupt Service Routine. Source - SAR ADC.
*
******************************************************************************/
CY_ISR(ADC_COS_ISR_LOC)
{
    int16 mLUT_cos;
    int16 mLUT_sin;
    uint16 phi_0;
    uint32 sin_shift;
    uint16 v1_fi;
    uint16 v_LUT, phi_LUT_u,phi_LUT_o, phi;
    
    mLUT_cos = CY_GET_REG16(ADC_COS_SAR_WRK0_PTR)-2048;
    mLUT_sin = CY_GET_REG16(ADC_SIN_SAR_WRK0_PTR)-2048;
    // Simulating constant input values
   // mLUT_cos = 1800;
    //mLUT_sin = 100;
    voltCount_cos = mLUT_cos;
    voltCount_sin = mLUT_sin;
    OCTANTIFY(mLUT_cos,mLUT_sin, phi_0);
	phi_0 *= BRAD_PI >> 2;
	sin_shift =((uint32_t) (mLUT_sin)) * (uint32_t) ATAN_ONE;
	v1_fi =(uint16_t) (sin_shift / mLUT_cos);
    v_LUT = v1_fi >> 5;
	phi_LUT_u = A[v_LUT];
	phi_LUT_o = A[v_LUT + 1];
	// Interpolation
	phi = ((((phi_LUT_o - phi_LUT_u) * (v1_fi - v_LUT * 32)) / 32 
	+ phi_LUT_u) >> 3) + phi_0; 
    // TP-Filterung (3 / 4 neu und 1 / 4 alt)
    phi_LUT_int = (phi+phi_alt) >> 1;
    phi_alt = phi;
    // Ausgabe zur AZ-Bestimmung
    phi_LUT_u=  CY_GET_REG8(&phi_LUT_int)&(0x7F);
    phi_mod_Control=phi_LUT_u;
    Pin_Period_DR  = (Pin_Period_DR & 0xBF)| period;
    
    if (period == 0) 
      period = 64;  // gleich richtige Bit setzen
    else 
      period = 0;
    
}


/******************************************************************************
* Function Name: DISP_ISR_LOC
*******************************************************************************
*
* Summary:
*  Handle Interrupt Service Routine. Source - Clock2.
*
******************************************************************************/
CY_ISR(DISP_ISR_LOC)
{
  Disp_Flag = 1;  
}

int main(void)
{
    int result;
    uint32 phi_disp;
	char displayStr[15] = {'\0'};
       
    /* Start ADC and start conversion */
    ADC_COS_Start();
     //  ADC_COS_StartConvert();
    
    ADC_SIN_Start();
    //   ADC_SIN_StartConvert();
   
    /* Start LCD and set position */
    LCD_Start();
       
    PGA_1_Start();
    PGA_2_Start();
    PGA_3_Start();
    PGA_4_Start();
    
    ADC_COS_IRQ_StartEx(ADC_COS_ISR_LOC);
       
    isr_disp_StartEx(DISP_ISR_LOC);
    /* Enable Global Interrupts */
     CyGlobalIntEnable;
        
    while(1)
    {
                 
        if (Disp_Flag == 1)
        {
            // Doing nothing for testing!
            
            phi_disp = (phi_LUT_int * (uint64)360000) >> 16; // für P = 2
                    
            sprintf(displayStr,"%4d %4d ",(int)voltCount_sin,(int)voltCount_cos);
            LCD_Position(0,0);
            LCD_PrintString(displayStr);
            result = sprintf(displayStr,"%6d mGrad",(int)phi_disp);
            LCD_Position(1,0);
            LCD_PrintString(displayStr);
            Disp_Flag = 0;
        }
    	
    }
}
/* [] END OF FILE */


